window.Gleam.initSearch([{"doc":"gladvent","title":"gladvent","content":"# Gladvent\n\n[![Hex Package](https://img.shields.io/hexpm/v/gladvent?color=ffaff3&amp;label=%F0%9F%93%A6)](https://hex.pm/packages/gladvent)\n[![Hex.pm](https://img.shields.io/hexpm/dt/gladvent?color=ffaff3)](https://hex.pm/packages/gladvent)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3?label=%F0%9F%93%9A)](https://hexdocs.pm/gladvent/)\n[![GitHub Workflow Status](https://img.shields.io/github/workflow/status/tanklesxl/gladvent/main)](https://github.com/tanklesxl/gladvent/actions)\n\nAn Advent Of Code runner for Gleam\n\nThis library is intended to be imported to your gleam project and used as a command runner for your advent of code project in gleam.\n\nTo add this library to your project run: `gleam add gladvent` and add `import gladvent` to your main gleam file.\n\n&gt; [!IMPORTANT]\n&gt; This package works only on gleam&#39;s erlang target!\n\n&gt; [!NOTE]\n&gt; Due to changes made in gleam 1.5, users that were calling gladvent via `gleam run -m` should upgrade to v2 and add a call to `gladvent.run` in their project `main` function.\n\n## Using the library\n\nOnce you&#39;ve added gladvent as a dependency via `gleam add gladvent`, simply add `gladvent.run()` to your project&#39;s `main` function.\n\n## Multi-year support\n\nGladvent now comes with out-of-the-box multi-year support via the `--year` flag when running it.\n\nFor convenience it defaults to the current year. Therefore, passing `--year=YEAR` to either the `run`, `run all` or `new` commands will use the year specified or the current year if the flag was not provided.\n\n## Seeing help messages\n\n- To see available subcommands: `gleam run -- --help`\n- To see help for the `run` command: `gleam run run --help`\n- To see help for the `run` command: `gleam run run all --help`\n- To see help for the `new` command: `gleam run new --help`\n\n## General Workflow\n\nWhere X is the day you&#39;d like to add:\n\n_Note:_ this method requires all day solutions be in `src/aoc_&lt;year&gt;/` with filenames `day_X.gleam`, each solution module containing `fn pt_1(String) -&gt; Int` and a `fn pt_2(String) -&gt; Int`\n\n1. run `gleam run new X`\n2. add your input to `input/&lt;YEAR&gt;/day_X.txt`\n3. add your code to `src/aoc_&lt;YEAR&gt;/day_X.gleam`\n4. run `gleam run run X`\n\n### Available commands\n\nThis project provides your application with 2 command groups, `new` and `run`:\n\n#### New\n\n- `new`: create `src/aoc_&lt;year&gt;/day_&lt;day&gt;.gleam` and `input/&lt;year&gt;/&lt;day&gt;.txt` files that correspond to the specified days\n  - format: `gleam run new a b c ...`\n\n#### Run\n\nThe `run` command expects input files to be in the `input/&lt;year&gt;` directory, and code to be in `src/aoc_&lt;year&gt;/`\n(corresponding to the files created by the `new` command).\n\n- `run`: run the specified days\n  - format: `gleam run run a b c ...`\n\n- `run all`: run all registered days\n  - format: `gleam run run all`\n\n_Note:_\n\n- any triggered `assert`,`panic` or `todo` will be captured and printed, for example:\n\n```\nPart 1: error: todo - unimplemented in module aoc_2024/day_1 in function pt_1 at line 2\n```\n\n\n## Reusable parse funtions\n\nGladvent supports modules with functions that provide a `pub fn parse(String) -&gt; a` where the type `a` matches with the type of the argument for the runner functions `pt_1` and `pt_2`.\nIf this `parse` function is present, gladvent will pick it up and run it only once, providing the output to both runner functions.\n\nAn example of which looks like this:\n\n```gleam\npub fn parse(input: String) -&gt; Int {\n    let assert Ok(i) = int.parse(input)\n    i\n}\n\npub fn pt_1(input: Int) -&gt; Int {\n    i + 1\n}\n\npub fn pt_2(input: Int) -&gt; Int {\n    i + 2\n}\n```\n\n_Note_: `gladvent` now leverages gleam&#39;s `export package-interface` functionality to type-check your `parse` and `pt_{1|2}` functions to make sure that they are compatible with each other.\n\n## Defining expectations for easy refactoring\n\nOne of the most satisfying aspects of advent of code (for me), second only to that sweet feeling of first solving a problem, is *iteration and refactoring*.\n\nGladvent makes it easy for you to define expected outputs in your `gleam.toml` for all your solutions so that you can have the confidence to refactor your solutions as much as you want without having to constantly compare with your submissions on the advent of code website..\n\n### Expectations in `gleam.toml`\n\nDefining expectations is as simple as adding sections to your `gleam.toml` in the following format:\n\n```toml\n[gladvent.&lt;year as int&gt;.&lt;day as int&gt;]\npt_1 = &lt;int or string&gt;\npt_2 = &lt;int or string&gt;\n```\n\nFor example, to set the expectations for Dec 1st 2024 (2024 day 1) you would add something like:\n\n```toml\n[gladvent.2024.1]\npt_1 = 1\npt_2 = 2\n```\n\nWhen running, gladvent will detect whether a specific day has it&#39;s expectations set and if so will print out the result for you.\n\nLet&#39;s say that your computed solution for 2024 day 1 is actually 1 for pt\\_1 and 3 for pt\\_2, the output will look like this:\n\n```\nRan 2024 day 1:\n  Part 1: ✅ met expected value: 1\n  Part 2: ❌ unmet expectation: got 3, expected 2\n```\n\n## Example inputs\n\nSometimes it&#39;s helpful to run advent of code solutions against example inputs to verify expectations.\nGladvent now provides a `--example` flag in both the `new` and `run` commands to conveniently support that workflow without needing to modify your actual problem input files.\nExample input files will be generated at and run from `input/&lt;year&gt;/&lt;day&gt;.example.txt`.\n\n_Note_: gladvent will not compare your solution output against the expectations defined in `gleam.toml` when running in example mode.\n\n## FAQ\n\n### Why did you make this?\n\nIt seemed fun, I like small command line utilities and I wanted a way to get advent of code done in gleam without having the additional overhead of lots of copy-pasting and connecting things to get it to run\n\n### Why does this not download the input from the advent of code website?\n\nA few reasons:\n\n1. I wanted to keep this utility as simple as possible to start with\n2. I like the advent of code website and I felt like it was a shame to circumvent visiting it, especially since you should access it to read the daily challenge. On top of that, I would like to avoid spamming the `advent of code` api if possible.\n\n### Why run as a command line utility and not just use unit tests?\n\nI thought a lot about that and I just prefer the overall interactivity of a CLI better, as well as allowing for endless runs or runs with configurable timeouts.\nHaving it run as part of `eunit` doesnt provide as much flexibility as I would like. Other testing frameworks have been popping up but I leave the decision to use them up to you!\n","url":"index.html"},{"doc":"gladvent","title":"run","content":"pub fn run() -&gt; Nil\n Add this function to your project&#39;s `main` function in order to run the gladvent CLI.\n\n This function gets its input from the command line arguments by using the `argv` library.\n\nSynonyms:\ngladvent.run\ngladvent run","url":"gladvent.html#run"},{"doc":"gladvent","title":"gladvent","content":"","url":"gladvent.html"}]);